#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 4
#endif
#ifndef PFXDIST
   #define PFXDIST 256
#endif
#ifndef PFADIST
   #define PFADIST 256
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
/* Need to handle BETA=0 case by assigning y to zero outside of loop */
void ATL_UGEMV(ATL_CINT M0, ATL_CINT N, const TYPE *A, ATL_CINT lda0,
               const TYPE *X, TYPE *Y)
/*
 * 24x4 unrolled mvn_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N4=(N/4)*4, M=M0+M0, M24=((M0/24)*24)<<1,
            lda=lda0+lda0, lda4=lda*4;
   ATL_INT j;

   #ifdef BETA0
      for (j=0; j < M; j++)
         Y[j] = ATL_rzero;
   #endif
   for (j=N4; j; j -= 4, A += lda4, X += 8)
   {
      const float *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda;
      register float rx0=X[0], ix0=X[1], rx1=X[2], ix1=X[3], rx2=X[4], ix2=X[5],
                     rx3=X[6], ix3=X[7];
      ATL_INT i;
      prefY(X+8+8-1);
      for (i=0; i < M24; i += 48)
      {
         const float ra0_0=A0[i+0], ia0_0=A0[i+1], ra1_0=A0[i+2], ia1_0=A0[i+3],
                     ra2_0=A0[i+4], ia2_0=A0[i+5], ra3_0=A0[i+6], ia3_0=A0[i+7],
                     ra4_0=A0[i+8], ia4_0=A0[i+9], ra5_0=A0[i+10],
                     ia5_0=A0[i+11], ra6_0=A0[i+12], ia6_0=A0[i+13],
                     ra7_0=A0[i+14], ia7_0=A0[i+15], ra8_0=A0[i+16],
                     ia8_0=A0[i+17], ra9_0=A0[i+18], ia9_0=A0[i+19],
                     ra10_0=A0[i+20], ia10_0=A0[i+21], ra11_0=A0[i+22],
                     ia11_0=A0[i+23], ra12_0=A0[i+24], ia12_0=A0[i+25],
                     ra13_0=A0[i+26], ia13_0=A0[i+27], ra14_0=A0[i+28],
                     ia14_0=A0[i+29], ra15_0=A0[i+30], ia15_0=A0[i+31],
                     ra16_0=A0[i+32], ia16_0=A0[i+33], ra17_0=A0[i+34],
                     ia17_0=A0[i+35], ra18_0=A0[i+36], ia18_0=A0[i+37],
                     ra19_0=A0[i+38], ia19_0=A0[i+39], ra20_0=A0[i+40],
                     ia20_0=A0[i+41], ra21_0=A0[i+42], ia21_0=A0[i+43],
                     ra22_0=A0[i+44], ia22_0=A0[i+45], ra23_0=A0[i+46],
                     ia23_0=A0[i+47], ra0_1=A1[i+0], ia0_1=A1[i+1],
                     ra1_1=A1[i+2], ia1_1=A1[i+3], ra2_1=A1[i+4], ia2_1=A1[i+5],
                     ra3_1=A1[i+6], ia3_1=A1[i+7], ra4_1=A1[i+8], ia4_1=A1[i+9],
                     ra5_1=A1[i+10], ia5_1=A1[i+11], ra6_1=A1[i+12],
                     ia6_1=A1[i+13], ra7_1=A1[i+14], ia7_1=A1[i+15],
                     ra8_1=A1[i+16], ia8_1=A1[i+17], ra9_1=A1[i+18],
                     ia9_1=A1[i+19], ra10_1=A1[i+20], ia10_1=A1[i+21],
                     ra11_1=A1[i+22], ia11_1=A1[i+23], ra12_1=A1[i+24],
                     ia12_1=A1[i+25], ra13_1=A1[i+26], ia13_1=A1[i+27],
                     ra14_1=A1[i+28], ia14_1=A1[i+29], ra15_1=A1[i+30],
                     ia15_1=A1[i+31], ra16_1=A1[i+32], ia16_1=A1[i+33],
                     ra17_1=A1[i+34], ia17_1=A1[i+35], ra18_1=A1[i+36],
                     ia18_1=A1[i+37], ra19_1=A1[i+38], ia19_1=A1[i+39],
                     ra20_1=A1[i+40], ia20_1=A1[i+41], ra21_1=A1[i+42],
                     ia21_1=A1[i+43], ra22_1=A1[i+44], ia22_1=A1[i+45],
                     ra23_1=A1[i+46], ia23_1=A1[i+47], ra0_2=A2[i+0],
                     ia0_2=A2[i+1], ra1_2=A2[i+2], ia1_2=A2[i+3], ra2_2=A2[i+4],
                     ia2_2=A2[i+5], ra3_2=A2[i+6], ia3_2=A2[i+7], ra4_2=A2[i+8],
                     ia4_2=A2[i+9], ra5_2=A2[i+10], ia5_2=A2[i+11],
                     ra6_2=A2[i+12], ia6_2=A2[i+13], ra7_2=A2[i+14],
                     ia7_2=A2[i+15], ra8_2=A2[i+16], ia8_2=A2[i+17],
                     ra9_2=A2[i+18], ia9_2=A2[i+19], ra10_2=A2[i+20],
                     ia10_2=A2[i+21], ra11_2=A2[i+22], ia11_2=A2[i+23],
                     ra12_2=A2[i+24], ia12_2=A2[i+25], ra13_2=A2[i+26],
                     ia13_2=A2[i+27], ra14_2=A2[i+28], ia14_2=A2[i+29],
                     ra15_2=A2[i+30], ia15_2=A2[i+31], ra16_2=A2[i+32],
                     ia16_2=A2[i+33], ra17_2=A2[i+34], ia17_2=A2[i+35],
                     ra18_2=A2[i+36], ia18_2=A2[i+37], ra19_2=A2[i+38],
                     ia19_2=A2[i+39], ra20_2=A2[i+40], ia20_2=A2[i+41],
                     ra21_2=A2[i+42], ia21_2=A2[i+43], ra22_2=A2[i+44],
                     ia22_2=A2[i+45], ra23_2=A2[i+46], ia23_2=A2[i+47],
                     ra0_3=A3[i+0], ia0_3=A3[i+1], ra1_3=A3[i+2], ia1_3=A3[i+3],
                     ra2_3=A3[i+4], ia2_3=A3[i+5], ra3_3=A3[i+6], ia3_3=A3[i+7],
                     ra4_3=A3[i+8], ia4_3=A3[i+9], ra5_3=A3[i+10],
                     ia5_3=A3[i+11], ra6_3=A3[i+12], ia6_3=A3[i+13],
                     ra7_3=A3[i+14], ia7_3=A3[i+15], ra8_3=A3[i+16],
                     ia8_3=A3[i+17], ra9_3=A3[i+18], ia9_3=A3[i+19],
                     ra10_3=A3[i+20], ia10_3=A3[i+21], ra11_3=A3[i+22],
                     ia11_3=A3[i+23], ra12_3=A3[i+24], ia12_3=A3[i+25],
                     ra13_3=A3[i+26], ia13_3=A3[i+27], ra14_3=A3[i+28],
                     ia14_3=A3[i+29], ra15_3=A3[i+30], ia15_3=A3[i+31],
                     ra16_3=A3[i+32], ia16_3=A3[i+33], ra17_3=A3[i+34],
                     ia17_3=A3[i+35], ra18_3=A3[i+36], ia18_3=A3[i+37],
                     ra19_3=A3[i+38], ia19_3=A3[i+39], ra20_3=A3[i+40],
                     ia20_3=A3[i+41], ra21_3=A3[i+42], ia21_3=A3[i+43],
                     ra22_3=A3[i+44], ia22_3=A3[i+45], ra23_3=A3[i+46],
                     ia23_3=A3[i+47];
         register float ry0=Y[i+0], iy0=Y[i+1], ry1=Y[i+2], iy1=Y[i+3],
                        ry2=Y[i+4], iy2=Y[i+5], ry3=Y[i+6], iy3=Y[i+7],
                        ry4=Y[i+8], iy4=Y[i+9], ry5=Y[i+10], iy5=Y[i+11],
                        ry6=Y[i+12], iy6=Y[i+13], ry7=Y[i+14], iy7=Y[i+15],
                        ry8=Y[i+16], iy8=Y[i+17], ry9=Y[i+18], iy9=Y[i+19],
                        ry10=Y[i+20], iy10=Y[i+21], ry11=Y[i+22], iy11=Y[i+23],
                        ry12=Y[i+24], iy12=Y[i+25], ry13=Y[i+26], iy13=Y[i+27],
                        ry14=Y[i+28], iy14=Y[i+29], ry15=Y[i+30], iy15=Y[i+31],
                        ry16=Y[i+32], iy16=Y[i+33], ry17=Y[i+34], iy17=Y[i+35],
                        ry18=Y[i+36], iy18=Y[i+37], ry19=Y[i+38], iy19=Y[i+39],
                        ry20=Y[i+40], iy20=Y[i+41], ry21=Y[i+42], iy21=Y[i+43],
                        ry22=Y[i+44], iy22=Y[i+45], ry23=Y[i+46], iy23=Y[i+47];

         prefY(Y);
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry1 += ra1_0 * rx0;
         iy1 += ra1_0 * ix0;
         ry2 += ra2_0 * rx0;
         iy2 += ra2_0 * ix0;
         ry3 += ra3_0 * rx0;
         iy3 += ra3_0 * ix0;
         ry4 += ra4_0 * rx0;
         iy4 += ra4_0 * ix0;
         ry5 += ra5_0 * rx0;
         iy5 += ra5_0 * ix0;
         ry6 += ra6_0 * rx0;
         iy6 += ra6_0 * ix0;
         ry7 += ra7_0 * rx0;
         iy7 += ra7_0 * ix0;
         ry8 += ra8_0 * rx0;
         iy8 += ra8_0 * ix0;
         ry9 += ra9_0 * rx0;
         iy9 += ra9_0 * ix0;
         ry10 += ra10_0 * rx0;
         iy10 += ra10_0 * ix0;
         ry11 += ra11_0 * rx0;
         iy11 += ra11_0 * ix0;
         ry12 += ra12_0 * rx0;
         iy12 += ra12_0 * ix0;
         ry13 += ra13_0 * rx0;
         iy13 += ra13_0 * ix0;
         ry14 += ra14_0 * rx0;
         iy14 += ra14_0 * ix0;
         ry15 += ra15_0 * rx0;
         iy15 += ra15_0 * ix0;
         ry16 += ra16_0 * rx0;
         iy16 += ra16_0 * ix0;
         ry17 += ra17_0 * rx0;
         iy17 += ra17_0 * ix0;
         ry18 += ra18_0 * rx0;
         iy18 += ra18_0 * ix0;
         ry19 += ra19_0 * rx0;
         iy19 += ra19_0 * ix0;
         ry20 += ra20_0 * rx0;
         iy20 += ra20_0 * ix0;
         ry21 += ra21_0 * rx0;
         iy21 += ra21_0 * ix0;
         ry22 += ra22_0 * rx0;
         iy22 += ra22_0 * ix0;
         ry23 += ra23_0 * rx0;
         iy23 += ra23_0 * ix0;
         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         prefA(A0);
         ry1 -= ia1_0 * ix0;
         iy1 += ia1_0 * rx0;
         ry2 -= ia2_0 * ix0;
         iy2 += ia2_0 * rx0;
         ry3 -= ia3_0 * ix0;
         iy3 += ia3_0 * rx0;
         ry4 -= ia4_0 * ix0;
         iy4 += ia4_0 * rx0;
         ry5 -= ia5_0 * ix0;
         iy5 += ia5_0 * rx0;
         ry6 -= ia6_0 * ix0;
         iy6 += ia6_0 * rx0;
         ry7 -= ia7_0 * ix0;
         iy7 += ia7_0 * rx0;
         ry8 -= ia8_0 * ix0;
         iy8 += ia8_0 * rx0;
         prefA(A0);
         ry9 -= ia9_0 * ix0;
         iy9 += ia9_0 * rx0;
         ry10 -= ia10_0 * ix0;
         iy10 += ia10_0 * rx0;
         ry11 -= ia11_0 * ix0;
         iy11 += ia11_0 * rx0;
         ry12 -= ia12_0 * ix0;
         iy12 += ia12_0 * rx0;
         ry13 -= ia13_0 * ix0;
         iy13 += ia13_0 * rx0;
         ry14 -= ia14_0 * ix0;
         iy14 += ia14_0 * rx0;
         ry15 -= ia15_0 * ix0;
         iy15 += ia15_0 * rx0;
         ry16 -= ia16_0 * ix0;
         iy16 += ia16_0 * rx0;
         prefA(A0);
         ry17 -= ia17_0 * ix0;
         iy17 += ia17_0 * rx0;
         ry18 -= ia18_0 * ix0;
         iy18 += ia18_0 * rx0;
         ry19 -= ia19_0 * ix0;
         iy19 += ia19_0 * rx0;
         ry20 -= ia20_0 * ix0;
         iy20 += ia20_0 * rx0;
         ry21 -= ia21_0 * ix0;
         iy21 += ia21_0 * rx0;
         ry22 -= ia22_0 * ix0;
         iy22 += ia22_0 * rx0;
         ry23 -= ia23_0 * ix0;
         iy23 += ia23_0 * rx0;

         ry0 += ra0_1 * rx1;
         iy0 += ra0_1 * ix1;
         ry1 += ra1_1 * rx1;
         iy1 += ra1_1 * ix1;
         ry2 += ra2_1 * rx1;
         iy2 += ra2_1 * ix1;
         ry3 += ra3_1 * rx1;
         iy3 += ra3_1 * ix1;
         ry4 += ra4_1 * rx1;
         iy4 += ra4_1 * ix1;
         ry5 += ra5_1 * rx1;
         iy5 += ra5_1 * ix1;
         ry6 += ra6_1 * rx1;
         iy6 += ra6_1 * ix1;
         ry7 += ra7_1 * rx1;
         iy7 += ra7_1 * ix1;
         ry8 += ra8_1 * rx1;
         iy8 += ra8_1 * ix1;
         ry9 += ra9_1 * rx1;
         iy9 += ra9_1 * ix1;
         ry10 += ra10_1 * rx1;
         iy10 += ra10_1 * ix1;
         ry11 += ra11_1 * rx1;
         iy11 += ra11_1 * ix1;
         ry12 += ra12_1 * rx1;
         iy12 += ra12_1 * ix1;
         ry13 += ra13_1 * rx1;
         iy13 += ra13_1 * ix1;
         ry14 += ra14_1 * rx1;
         iy14 += ra14_1 * ix1;
         ry15 += ra15_1 * rx1;
         iy15 += ra15_1 * ix1;
         ry16 += ra16_1 * rx1;
         iy16 += ra16_1 * ix1;
         ry17 += ra17_1 * rx1;
         iy17 += ra17_1 * ix1;
         ry18 += ra18_1 * rx1;
         iy18 += ra18_1 * ix1;
         ry19 += ra19_1 * rx1;
         iy19 += ra19_1 * ix1;
         ry20 += ra20_1 * rx1;
         iy20 += ra20_1 * ix1;
         ry21 += ra21_1 * rx1;
         iy21 += ra21_1 * ix1;
         ry22 += ra22_1 * rx1;
         iy22 += ra22_1 * ix1;
         ry23 += ra23_1 * rx1;
         iy23 += ra23_1 * ix1;
         ry0 -= ia0_1 * ix1;
         iy0 += ia0_1 * rx1;
         prefA(A1);
         ry1 -= ia1_1 * ix1;
         iy1 += ia1_1 * rx1;
         ry2 -= ia2_1 * ix1;
         iy2 += ia2_1 * rx1;
         ry3 -= ia3_1 * ix1;
         iy3 += ia3_1 * rx1;
         ry4 -= ia4_1 * ix1;
         iy4 += ia4_1 * rx1;
         ry5 -= ia5_1 * ix1;
         iy5 += ia5_1 * rx1;
         ry6 -= ia6_1 * ix1;
         iy6 += ia6_1 * rx1;
         ry7 -= ia7_1 * ix1;
         iy7 += ia7_1 * rx1;
         ry8 -= ia8_1 * ix1;
         iy8 += ia8_1 * rx1;
         prefA(A1);
         ry9 -= ia9_1 * ix1;
         iy9 += ia9_1 * rx1;
         ry10 -= ia10_1 * ix1;
         iy10 += ia10_1 * rx1;
         ry11 -= ia11_1 * ix1;
         iy11 += ia11_1 * rx1;
         ry12 -= ia12_1 * ix1;
         iy12 += ia12_1 * rx1;
         ry13 -= ia13_1 * ix1;
         iy13 += ia13_1 * rx1;
         ry14 -= ia14_1 * ix1;
         iy14 += ia14_1 * rx1;
         ry15 -= ia15_1 * ix1;
         iy15 += ia15_1 * rx1;
         ry16 -= ia16_1 * ix1;
         iy16 += ia16_1 * rx1;
         prefA(A1);
         ry17 -= ia17_1 * ix1;
         iy17 += ia17_1 * rx1;
         ry18 -= ia18_1 * ix1;
         iy18 += ia18_1 * rx1;
         ry19 -= ia19_1 * ix1;
         iy19 += ia19_1 * rx1;
         ry20 -= ia20_1 * ix1;
         iy20 += ia20_1 * rx1;
         ry21 -= ia21_1 * ix1;
         iy21 += ia21_1 * rx1;
         ry22 -= ia22_1 * ix1;
         iy22 += ia22_1 * rx1;
         ry23 -= ia23_1 * ix1;
         iy23 += ia23_1 * rx1;

         ry0 += ra0_2 * rx2;
         iy0 += ra0_2 * ix2;
         ry1 += ra1_2 * rx2;
         iy1 += ra1_2 * ix2;
         ry2 += ra2_2 * rx2;
         iy2 += ra2_2 * ix2;
         ry3 += ra3_2 * rx2;
         iy3 += ra3_2 * ix2;
         ry4 += ra4_2 * rx2;
         iy4 += ra4_2 * ix2;
         ry5 += ra5_2 * rx2;
         iy5 += ra5_2 * ix2;
         ry6 += ra6_2 * rx2;
         iy6 += ra6_2 * ix2;
         ry7 += ra7_2 * rx2;
         iy7 += ra7_2 * ix2;
         ry8 += ra8_2 * rx2;
         iy8 += ra8_2 * ix2;
         ry9 += ra9_2 * rx2;
         iy9 += ra9_2 * ix2;
         ry10 += ra10_2 * rx2;
         iy10 += ra10_2 * ix2;
         ry11 += ra11_2 * rx2;
         iy11 += ra11_2 * ix2;
         ry12 += ra12_2 * rx2;
         iy12 += ra12_2 * ix2;
         ry13 += ra13_2 * rx2;
         iy13 += ra13_2 * ix2;
         ry14 += ra14_2 * rx2;
         iy14 += ra14_2 * ix2;
         ry15 += ra15_2 * rx2;
         iy15 += ra15_2 * ix2;
         ry16 += ra16_2 * rx2;
         iy16 += ra16_2 * ix2;
         ry17 += ra17_2 * rx2;
         iy17 += ra17_2 * ix2;
         ry18 += ra18_2 * rx2;
         iy18 += ra18_2 * ix2;
         ry19 += ra19_2 * rx2;
         iy19 += ra19_2 * ix2;
         ry20 += ra20_2 * rx2;
         iy20 += ra20_2 * ix2;
         ry21 += ra21_2 * rx2;
         iy21 += ra21_2 * ix2;
         ry22 += ra22_2 * rx2;
         iy22 += ra22_2 * ix2;
         ry23 += ra23_2 * rx2;
         iy23 += ra23_2 * ix2;
         ry0 -= ia0_2 * ix2;
         iy0 += ia0_2 * rx2;
         prefA(A2);
         ry1 -= ia1_2 * ix2;
         iy1 += ia1_2 * rx2;
         ry2 -= ia2_2 * ix2;
         iy2 += ia2_2 * rx2;
         ry3 -= ia3_2 * ix2;
         iy3 += ia3_2 * rx2;
         ry4 -= ia4_2 * ix2;
         iy4 += ia4_2 * rx2;
         ry5 -= ia5_2 * ix2;
         iy5 += ia5_2 * rx2;
         ry6 -= ia6_2 * ix2;
         iy6 += ia6_2 * rx2;
         ry7 -= ia7_2 * ix2;
         iy7 += ia7_2 * rx2;
         ry8 -= ia8_2 * ix2;
         iy8 += ia8_2 * rx2;
         prefA(A2);
         ry9 -= ia9_2 * ix2;
         iy9 += ia9_2 * rx2;
         ry10 -= ia10_2 * ix2;
         iy10 += ia10_2 * rx2;
         ry11 -= ia11_2 * ix2;
         iy11 += ia11_2 * rx2;
         ry12 -= ia12_2 * ix2;
         iy12 += ia12_2 * rx2;
         ry13 -= ia13_2 * ix2;
         iy13 += ia13_2 * rx2;
         ry14 -= ia14_2 * ix2;
         iy14 += ia14_2 * rx2;
         ry15 -= ia15_2 * ix2;
         iy15 += ia15_2 * rx2;
         ry16 -= ia16_2 * ix2;
         iy16 += ia16_2 * rx2;
         prefA(A2);
         ry17 -= ia17_2 * ix2;
         iy17 += ia17_2 * rx2;
         ry18 -= ia18_2 * ix2;
         iy18 += ia18_2 * rx2;
         ry19 -= ia19_2 * ix2;
         iy19 += ia19_2 * rx2;
         ry20 -= ia20_2 * ix2;
         iy20 += ia20_2 * rx2;
         ry21 -= ia21_2 * ix2;
         iy21 += ia21_2 * rx2;
         ry22 -= ia22_2 * ix2;
         iy22 += ia22_2 * rx2;
         ry23 -= ia23_2 * ix2;
         iy23 += ia23_2 * rx2;

         ry0 += ra0_3 * rx3;
         iy0 += ra0_3 * ix3;
         ry1 += ra1_3 * rx3;
         iy1 += ra1_3 * ix3;
         ry2 += ra2_3 * rx3;
         iy2 += ra2_3 * ix3;
         ry3 += ra3_3 * rx3;
         iy3 += ra3_3 * ix3;
         ry4 += ra4_3 * rx3;
         iy4 += ra4_3 * ix3;
         ry5 += ra5_3 * rx3;
         iy5 += ra5_3 * ix3;
         ry6 += ra6_3 * rx3;
         iy6 += ra6_3 * ix3;
         ry7 += ra7_3 * rx3;
         iy7 += ra7_3 * ix3;
         ry8 += ra8_3 * rx3;
         iy8 += ra8_3 * ix3;
         ry9 += ra9_3 * rx3;
         iy9 += ra9_3 * ix3;
         ry10 += ra10_3 * rx3;
         iy10 += ra10_3 * ix3;
         ry11 += ra11_3 * rx3;
         iy11 += ra11_3 * ix3;
         ry12 += ra12_3 * rx3;
         iy12 += ra12_3 * ix3;
         ry13 += ra13_3 * rx3;
         iy13 += ra13_3 * ix3;
         ry14 += ra14_3 * rx3;
         iy14 += ra14_3 * ix3;
         ry15 += ra15_3 * rx3;
         iy15 += ra15_3 * ix3;
         ry16 += ra16_3 * rx3;
         iy16 += ra16_3 * ix3;
         ry17 += ra17_3 * rx3;
         iy17 += ra17_3 * ix3;
         ry18 += ra18_3 * rx3;
         iy18 += ra18_3 * ix3;
         ry19 += ra19_3 * rx3;
         iy19 += ra19_3 * ix3;
         ry20 += ra20_3 * rx3;
         iy20 += ra20_3 * ix3;
         ry21 += ra21_3 * rx3;
         iy21 += ra21_3 * ix3;
         ry22 += ra22_3 * rx3;
         iy22 += ra22_3 * ix3;
         ry23 += ra23_3 * rx3;
         iy23 += ra23_3 * ix3;
         ry0 -= ia0_3 * ix3;
         iy0 += ia0_3 * rx3;
         prefA(A3);
         ry1 -= ia1_3 * ix3;
         iy1 += ia1_3 * rx3;
         ry2 -= ia2_3 * ix3;
         iy2 += ia2_3 * rx3;
         ry3 -= ia3_3 * ix3;
         iy3 += ia3_3 * rx3;
         ry4 -= ia4_3 * ix3;
         iy4 += ia4_3 * rx3;
         ry5 -= ia5_3 * ix3;
         iy5 += ia5_3 * rx3;
         ry6 -= ia6_3 * ix3;
         iy6 += ia6_3 * rx3;
         ry7 -= ia7_3 * ix3;
         iy7 += ia7_3 * rx3;
         ry8 -= ia8_3 * ix3;
         iy8 += ia8_3 * rx3;
         prefA(A3);
         ry9 -= ia9_3 * ix3;
         iy9 += ia9_3 * rx3;
         ry10 -= ia10_3 * ix3;
         iy10 += ia10_3 * rx3;
         ry11 -= ia11_3 * ix3;
         iy11 += ia11_3 * rx3;
         ry12 -= ia12_3 * ix3;
         iy12 += ia12_3 * rx3;
         ry13 -= ia13_3 * ix3;
         iy13 += ia13_3 * rx3;
         ry14 -= ia14_3 * ix3;
         iy14 += ia14_3 * rx3;
         ry15 -= ia15_3 * ix3;
         iy15 += ia15_3 * rx3;
         ry16 -= ia16_3 * ix3;
         iy16 += ia16_3 * rx3;
         prefA(A3);
         ry17 -= ia17_3 * ix3;
         iy17 += ia17_3 * rx3;
         ry18 -= ia18_3 * ix3;
         iy18 += ia18_3 * rx3;
         ry19 -= ia19_3 * ix3;
         iy19 += ia19_3 * rx3;
         ry20 -= ia20_3 * ix3;
         iy20 += ia20_3 * rx3;
         ry21 -= ia21_3 * ix3;
         iy21 += ia21_3 * rx3;
         ry22 -= ia22_3 * ix3;
         iy22 += ia22_3 * rx3;
         ry23 -= ia23_3 * ix3;
         iy23 += ia23_3 * rx3;


         Y[i+0] = ry0;
         Y[i+1] = iy0;
         Y[i+2] = ry1;
         Y[i+3] = iy1;
         Y[i+4] = ry2;
         Y[i+5] = iy2;
         Y[i+6] = ry3;
         Y[i+7] = iy3;
         Y[i+8] = ry4;
         Y[i+9] = iy4;
         Y[i+10] = ry5;
         Y[i+11] = iy5;
         Y[i+12] = ry6;
         Y[i+13] = iy6;
         Y[i+14] = ry7;
         Y[i+15] = iy7;
         Y[i+16] = ry8;
         Y[i+17] = iy8;
         Y[i+18] = ry9;
         Y[i+19] = iy9;
         Y[i+20] = ry10;
         Y[i+21] = iy10;
         Y[i+22] = ry11;
         Y[i+23] = iy11;
         Y[i+24] = ry12;
         Y[i+25] = iy12;
         Y[i+26] = ry13;
         Y[i+27] = iy13;
         Y[i+28] = ry14;
         Y[i+29] = iy14;
         Y[i+30] = ry15;
         Y[i+31] = iy15;
         Y[i+32] = ry16;
         Y[i+33] = iy16;
         Y[i+34] = ry17;
         Y[i+35] = iy17;
         Y[i+36] = ry18;
         Y[i+37] = iy18;
         Y[i+38] = ry19;
         Y[i+39] = iy19;
         Y[i+40] = ry20;
         Y[i+41] = iy20;
         Y[i+42] = ry21;
         Y[i+43] = iy21;
         Y[i+44] = ry22;
         Y[i+45] = iy22;
         Y[i+46] = ry23;
         Y[i+47] = iy23;
      }
      for (i=M24; i < M; i += 2)
      {
         register float ry0 = Y[i], iy0 = Y[i+1];
         const float ra0_0=A0[i], ia0_0=A0[i+1], ra0_1=A1[i], ia0_1=A1[i+1],
                     ra0_2=A2[i], ia0_2=A2[i+1], ra0_3=A3[i], ia0_3=A3[i+1];
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         ry0 += ra0_1 * rx1;
         iy0 += ra0_1 * ix1;
         ry0 -= ia0_1 * ix1;
         iy0 += ia0_1 * rx1;
         ry0 += ra0_2 * rx2;
         iy0 += ra0_2 * ix2;
         ry0 -= ia0_2 * ix2;
         iy0 += ia0_2 * rx2;
         ry0 += ra0_3 * rx3;
         iy0 += ra0_3 * ix3;
         ry0 -= ia0_3 * ix3;
         iy0 += ia0_3 * rx3;
         Y[i] = ry0;
         Y[i+1] = iy0;
      }
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N4; j; j--, A += lda, X += 2)
   {
      const float *A0=A;
      register float rx0=X[0], ix0=X[1];
      ATL_INT i;
      prefY(X+8+8-1);
      for (i=0; i < M24; i += 48)
      {
         const float ra0_0=A0[i+0], ia0_0=A0[i+1], ra1_0=A0[i+2], ia1_0=A0[i+3],
                     ra2_0=A0[i+4], ia2_0=A0[i+5], ra3_0=A0[i+6], ia3_0=A0[i+7],
                     ra4_0=A0[i+8], ia4_0=A0[i+9], ra5_0=A0[i+10],
                     ia5_0=A0[i+11], ra6_0=A0[i+12], ia6_0=A0[i+13],
                     ra7_0=A0[i+14], ia7_0=A0[i+15], ra8_0=A0[i+16],
                     ia8_0=A0[i+17], ra9_0=A0[i+18], ia9_0=A0[i+19],
                     ra10_0=A0[i+20], ia10_0=A0[i+21], ra11_0=A0[i+22],
                     ia11_0=A0[i+23], ra12_0=A0[i+24], ia12_0=A0[i+25],
                     ra13_0=A0[i+26], ia13_0=A0[i+27], ra14_0=A0[i+28],
                     ia14_0=A0[i+29], ra15_0=A0[i+30], ia15_0=A0[i+31],
                     ra16_0=A0[i+32], ia16_0=A0[i+33], ra17_0=A0[i+34],
                     ia17_0=A0[i+35], ra18_0=A0[i+36], ia18_0=A0[i+37],
                     ra19_0=A0[i+38], ia19_0=A0[i+39], ra20_0=A0[i+40],
                     ia20_0=A0[i+41], ra21_0=A0[i+42], ia21_0=A0[i+43],
                     ra22_0=A0[i+44], ia22_0=A0[i+45], ra23_0=A0[i+46],
                     ia23_0=A0[i+47];
         register float ry0=Y[i+0], iy0=Y[i+1], ry1=Y[i+2], iy1=Y[i+3],
                        ry2=Y[i+4], iy2=Y[i+5], ry3=Y[i+6], iy3=Y[i+7],
                        ry4=Y[i+8], iy4=Y[i+9], ry5=Y[i+10], iy5=Y[i+11],
                        ry6=Y[i+12], iy6=Y[i+13], ry7=Y[i+14], iy7=Y[i+15],
                        ry8=Y[i+16], iy8=Y[i+17], ry9=Y[i+18], iy9=Y[i+19],
                        ry10=Y[i+20], iy10=Y[i+21], ry11=Y[i+22], iy11=Y[i+23],
                        ry12=Y[i+24], iy12=Y[i+25], ry13=Y[i+26], iy13=Y[i+27],
                        ry14=Y[i+28], iy14=Y[i+29], ry15=Y[i+30], iy15=Y[i+31],
                        ry16=Y[i+32], iy16=Y[i+33], ry17=Y[i+34], iy17=Y[i+35],
                        ry18=Y[i+36], iy18=Y[i+37], ry19=Y[i+38], iy19=Y[i+39],
                        ry20=Y[i+40], iy20=Y[i+41], ry21=Y[i+42], iy21=Y[i+43],
                        ry22=Y[i+44], iy22=Y[i+45], ry23=Y[i+46], iy23=Y[i+47];

         prefY(Y);
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry1 += ra1_0 * rx0;
         iy1 += ra1_0 * ix0;
         ry2 += ra2_0 * rx0;
         iy2 += ra2_0 * ix0;
         ry3 += ra3_0 * rx0;
         iy3 += ra3_0 * ix0;
         ry4 += ra4_0 * rx0;
         iy4 += ra4_0 * ix0;
         ry5 += ra5_0 * rx0;
         iy5 += ra5_0 * ix0;
         ry6 += ra6_0 * rx0;
         iy6 += ra6_0 * ix0;
         ry7 += ra7_0 * rx0;
         iy7 += ra7_0 * ix0;
         ry8 += ra8_0 * rx0;
         iy8 += ra8_0 * ix0;
         ry9 += ra9_0 * rx0;
         iy9 += ra9_0 * ix0;
         ry10 += ra10_0 * rx0;
         iy10 += ra10_0 * ix0;
         ry11 += ra11_0 * rx0;
         iy11 += ra11_0 * ix0;
         ry12 += ra12_0 * rx0;
         iy12 += ra12_0 * ix0;
         ry13 += ra13_0 * rx0;
         iy13 += ra13_0 * ix0;
         ry14 += ra14_0 * rx0;
         iy14 += ra14_0 * ix0;
         ry15 += ra15_0 * rx0;
         iy15 += ra15_0 * ix0;
         ry16 += ra16_0 * rx0;
         iy16 += ra16_0 * ix0;
         ry17 += ra17_0 * rx0;
         iy17 += ra17_0 * ix0;
         ry18 += ra18_0 * rx0;
         iy18 += ra18_0 * ix0;
         ry19 += ra19_0 * rx0;
         iy19 += ra19_0 * ix0;
         ry20 += ra20_0 * rx0;
         iy20 += ra20_0 * ix0;
         ry21 += ra21_0 * rx0;
         iy21 += ra21_0 * ix0;
         ry22 += ra22_0 * rx0;
         iy22 += ra22_0 * ix0;
         ry23 += ra23_0 * rx0;
         iy23 += ra23_0 * ix0;
         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         prefA(A0);
         ry1 -= ia1_0 * ix0;
         iy1 += ia1_0 * rx0;
         ry2 -= ia2_0 * ix0;
         iy2 += ia2_0 * rx0;
         ry3 -= ia3_0 * ix0;
         iy3 += ia3_0 * rx0;
         ry4 -= ia4_0 * ix0;
         iy4 += ia4_0 * rx0;
         ry5 -= ia5_0 * ix0;
         iy5 += ia5_0 * rx0;
         ry6 -= ia6_0 * ix0;
         iy6 += ia6_0 * rx0;
         ry7 -= ia7_0 * ix0;
         iy7 += ia7_0 * rx0;
         ry8 -= ia8_0 * ix0;
         iy8 += ia8_0 * rx0;
         prefA(A0);
         ry9 -= ia9_0 * ix0;
         iy9 += ia9_0 * rx0;
         ry10 -= ia10_0 * ix0;
         iy10 += ia10_0 * rx0;
         ry11 -= ia11_0 * ix0;
         iy11 += ia11_0 * rx0;
         ry12 -= ia12_0 * ix0;
         iy12 += ia12_0 * rx0;
         ry13 -= ia13_0 * ix0;
         iy13 += ia13_0 * rx0;
         ry14 -= ia14_0 * ix0;
         iy14 += ia14_0 * rx0;
         ry15 -= ia15_0 * ix0;
         iy15 += ia15_0 * rx0;
         ry16 -= ia16_0 * ix0;
         iy16 += ia16_0 * rx0;
         prefA(A0);
         ry17 -= ia17_0 * ix0;
         iy17 += ia17_0 * rx0;
         ry18 -= ia18_0 * ix0;
         iy18 += ia18_0 * rx0;
         ry19 -= ia19_0 * ix0;
         iy19 += ia19_0 * rx0;
         ry20 -= ia20_0 * ix0;
         iy20 += ia20_0 * rx0;
         ry21 -= ia21_0 * ix0;
         iy21 += ia21_0 * rx0;
         ry22 -= ia22_0 * ix0;
         iy22 += ia22_0 * rx0;
         ry23 -= ia23_0 * ix0;
         iy23 += ia23_0 * rx0;


         Y[i+0] = ry0;
         Y[i+1] = iy0;
         Y[i+2] = ry1;
         Y[i+3] = iy1;
         Y[i+4] = ry2;
         Y[i+5] = iy2;
         Y[i+6] = ry3;
         Y[i+7] = iy3;
         Y[i+8] = ry4;
         Y[i+9] = iy4;
         Y[i+10] = ry5;
         Y[i+11] = iy5;
         Y[i+12] = ry6;
         Y[i+13] = iy6;
         Y[i+14] = ry7;
         Y[i+15] = iy7;
         Y[i+16] = ry8;
         Y[i+17] = iy8;
         Y[i+18] = ry9;
         Y[i+19] = iy9;
         Y[i+20] = ry10;
         Y[i+21] = iy10;
         Y[i+22] = ry11;
         Y[i+23] = iy11;
         Y[i+24] = ry12;
         Y[i+25] = iy12;
         Y[i+26] = ry13;
         Y[i+27] = iy13;
         Y[i+28] = ry14;
         Y[i+29] = iy14;
         Y[i+30] = ry15;
         Y[i+31] = iy15;
         Y[i+32] = ry16;
         Y[i+33] = iy16;
         Y[i+34] = ry17;
         Y[i+35] = iy17;
         Y[i+36] = ry18;
         Y[i+37] = iy18;
         Y[i+38] = ry19;
         Y[i+39] = iy19;
         Y[i+40] = ry20;
         Y[i+41] = iy20;
         Y[i+42] = ry21;
         Y[i+43] = iy21;
         Y[i+44] = ry22;
         Y[i+45] = iy22;
         Y[i+46] = ry23;
         Y[i+47] = iy23;
      }
      for (i=M24; i < M; i += 2)
      {
         register float ry0 = Y[i], iy0 = Y[i+1];
         const float ra0_0=A0[i], ia0_0=A0[i+1];
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         Y[i] = ry0;
         Y[i+1] = iy0;
      }
   }
}
