#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 12
#endif
#ifndef PFXDIST
   #define PFXDIST 256
#endif
#ifndef PFADIST
   #define PFADIST 256
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
void ATL_UGERK(ATL_CINT M0, ATL_CINT N, const float *X, const float *Y, 
               float *A, ATL_CINT lda0)
/*
 * 24x12 unrolled r1_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N12=(N/12)*12, M=M0+M0, M24=((M0/24)*24)<<1,
            lda=lda0+lda0, lda12=lda*12;
   ATL_INT j;

   for (j=N12; j; j -= 12, A += lda12, Y += 24)
   {
      float *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda, *A4=A3+lda, *A5=A4+lda,
            *A6=A5+lda, *A7=A6+lda, *A8=A7+lda, *A9=A8+lda, *A10=A9+lda,
            *A11=A10+lda;
      const register float ry0=Y[0], iy0=Y[1], ry1=Y[2], iy1=Y[3], ry2=Y[4],
                           iy2=Y[5], ry3=Y[6], iy3=Y[7], ry4=Y[8], iy4=Y[9],
                           ry5=Y[10], iy5=Y[11], ry6=Y[12], iy6=Y[13],
                           ry7=Y[14], iy7=Y[15], ry8=Y[16], iy8=Y[17],
                           ry9=Y[18], iy9=Y[19], ry10=Y[20], iy10=Y[21],
                           ry11=Y[22], iy11=Y[23];
      ATL_INT i;
      prefY(Y+24+24-1);
      for (i=0; i < M24; i += 48)
      {
         const register float rx0=X[i+0], ix0=X[i+1], rx1=X[i+2], ix1=X[i+3],
                              rx2=X[i+4], ix2=X[i+5], rx3=X[i+6], ix3=X[i+7],
                              rx4=X[i+8], ix4=X[i+9], rx5=X[i+10], ix5=X[i+11],
                              rx6=X[i+12], ix6=X[i+13], rx7=X[i+14],
                              ix7=X[i+15], rx8=X[i+16], ix8=X[i+17],
                              rx9=X[i+18], ix9=X[i+19], rx10=X[i+20],
                              ix10=X[i+21], rx11=X[i+22], ix11=X[i+23],
                              rx12=X[i+24], ix12=X[i+25], rx13=X[i+26],
                              ix13=X[i+27], rx14=X[i+28], ix14=X[i+29],
                              rx15=X[i+30], ix15=X[i+31], rx16=X[i+32],
                              ix16=X[i+33], rx17=X[i+34], ix17=X[i+35],
                              rx18=X[i+36], ix18=X[i+37], rx19=X[i+38],
                              ix19=X[i+39], rx20=X[i+40], ix20=X[i+41],
                              rx21=X[i+42], ix21=X[i+43], rx22=X[i+44],
                              ix22=X[i+45], rx23=X[i+46], ix23=X[i+47];

         prefX(X);
         A0[i+0] += ry0*rx0 - iy0*ix0;
         prefA(A0);
         A0[i+0+1] += ry0*ix0 + iy0*rx0;
         A1[i+0] += ry1*rx0 - iy1*ix0;
         prefA(A1);
         A1[i+0+1] += ry1*ix0 + iy1*rx0;
         A2[i+0] += ry2*rx0 - iy2*ix0;
         prefA(A2);
         A2[i+0+1] += ry2*ix0 + iy2*rx0;
         A3[i+0] += ry3*rx0 - iy3*ix0;
         prefA(A3);
         A3[i+0+1] += ry3*ix0 + iy3*rx0;
         A4[i+0] += ry4*rx0 - iy4*ix0;
         prefA(A4);
         A4[i+0+1] += ry4*ix0 + iy4*rx0;
         A5[i+0] += ry5*rx0 - iy5*ix0;
         prefA(A5);
         A5[i+0+1] += ry5*ix0 + iy5*rx0;
         A6[i+0] += ry6*rx0 - iy6*ix0;
         prefA(A6);
         A6[i+0+1] += ry6*ix0 + iy6*rx0;
         A7[i+0] += ry7*rx0 - iy7*ix0;
         prefA(A7);
         A7[i+0+1] += ry7*ix0 + iy7*rx0;
         A8[i+0] += ry8*rx0 - iy8*ix0;
         prefA(A8);
         A8[i+0+1] += ry8*ix0 + iy8*rx0;
         A9[i+0] += ry9*rx0 - iy9*ix0;
         prefA(A9);
         A9[i+0+1] += ry9*ix0 + iy9*rx0;
         A10[i+0] += ry10*rx0 - iy10*ix0;
         prefA(A10);
         A10[i+0+1] += ry10*ix0 + iy10*rx0;
         A11[i+0] += ry11*rx0 - iy11*ix0;
         prefA(A11);
         A11[i+0+1] += ry11*ix0 + iy11*rx0;
         A0[i+2] += ry0*rx1 - iy0*ix1;
         A0[i+2+1] += ry0*ix1 + iy0*rx1;
         A1[i+2] += ry1*rx1 - iy1*ix1;
         A1[i+2+1] += ry1*ix1 + iy1*rx1;
         A2[i+2] += ry2*rx1 - iy2*ix1;
         A2[i+2+1] += ry2*ix1 + iy2*rx1;
         A3[i+2] += ry3*rx1 - iy3*ix1;
         A3[i+2+1] += ry3*ix1 + iy3*rx1;
         A4[i+2] += ry4*rx1 - iy4*ix1;
         A4[i+2+1] += ry4*ix1 + iy4*rx1;
         A5[i+2] += ry5*rx1 - iy5*ix1;
         A5[i+2+1] += ry5*ix1 + iy5*rx1;
         A6[i+2] += ry6*rx1 - iy6*ix1;
         A6[i+2+1] += ry6*ix1 + iy6*rx1;
         A7[i+2] += ry7*rx1 - iy7*ix1;
         A7[i+2+1] += ry7*ix1 + iy7*rx1;
         A8[i+2] += ry8*rx1 - iy8*ix1;
         A8[i+2+1] += ry8*ix1 + iy8*rx1;
         A9[i+2] += ry9*rx1 - iy9*ix1;
         A9[i+2+1] += ry9*ix1 + iy9*rx1;
         A10[i+2] += ry10*rx1 - iy10*ix1;
         A10[i+2+1] += ry10*ix1 + iy10*rx1;
         A11[i+2] += ry11*rx1 - iy11*ix1;
         A11[i+2+1] += ry11*ix1 + iy11*rx1;
         A0[i+4] += ry0*rx2 - iy0*ix2;
         A0[i+4+1] += ry0*ix2 + iy0*rx2;
         A1[i+4] += ry1*rx2 - iy1*ix2;
         A1[i+4+1] += ry1*ix2 + iy1*rx2;
         A2[i+4] += ry2*rx2 - iy2*ix2;
         A2[i+4+1] += ry2*ix2 + iy2*rx2;
         A3[i+4] += ry3*rx2 - iy3*ix2;
         A3[i+4+1] += ry3*ix2 + iy3*rx2;
         A4[i+4] += ry4*rx2 - iy4*ix2;
         A4[i+4+1] += ry4*ix2 + iy4*rx2;
         A5[i+4] += ry5*rx2 - iy5*ix2;
         A5[i+4+1] += ry5*ix2 + iy5*rx2;
         A6[i+4] += ry6*rx2 - iy6*ix2;
         A6[i+4+1] += ry6*ix2 + iy6*rx2;
         A7[i+4] += ry7*rx2 - iy7*ix2;
         A7[i+4+1] += ry7*ix2 + iy7*rx2;
         A8[i+4] += ry8*rx2 - iy8*ix2;
         A8[i+4+1] += ry8*ix2 + iy8*rx2;
         A9[i+4] += ry9*rx2 - iy9*ix2;
         A9[i+4+1] += ry9*ix2 + iy9*rx2;
         A10[i+4] += ry10*rx2 - iy10*ix2;
         A10[i+4+1] += ry10*ix2 + iy10*rx2;
         A11[i+4] += ry11*rx2 - iy11*ix2;
         A11[i+4+1] += ry11*ix2 + iy11*rx2;
         A0[i+6] += ry0*rx3 - iy0*ix3;
         A0[i+6+1] += ry0*ix3 + iy0*rx3;
         A1[i+6] += ry1*rx3 - iy1*ix3;
         A1[i+6+1] += ry1*ix3 + iy1*rx3;
         A2[i+6] += ry2*rx3 - iy2*ix3;
         A2[i+6+1] += ry2*ix3 + iy2*rx3;
         A3[i+6] += ry3*rx3 - iy3*ix3;
         A3[i+6+1] += ry3*ix3 + iy3*rx3;
         A4[i+6] += ry4*rx3 - iy4*ix3;
         A4[i+6+1] += ry4*ix3 + iy4*rx3;
         A5[i+6] += ry5*rx3 - iy5*ix3;
         A5[i+6+1] += ry5*ix3 + iy5*rx3;
         A6[i+6] += ry6*rx3 - iy6*ix3;
         A6[i+6+1] += ry6*ix3 + iy6*rx3;
         A7[i+6] += ry7*rx3 - iy7*ix3;
         A7[i+6+1] += ry7*ix3 + iy7*rx3;
         A8[i+6] += ry8*rx3 - iy8*ix3;
         A8[i+6+1] += ry8*ix3 + iy8*rx3;
         A9[i+6] += ry9*rx3 - iy9*ix3;
         A9[i+6+1] += ry9*ix3 + iy9*rx3;
         A10[i+6] += ry10*rx3 - iy10*ix3;
         A10[i+6+1] += ry10*ix3 + iy10*rx3;
         A11[i+6] += ry11*rx3 - iy11*ix3;
         A11[i+6+1] += ry11*ix3 + iy11*rx3;
         A0[i+8] += ry0*rx4 - iy0*ix4;
         A0[i+8+1] += ry0*ix4 + iy0*rx4;
         A1[i+8] += ry1*rx4 - iy1*ix4;
         A1[i+8+1] += ry1*ix4 + iy1*rx4;
         A2[i+8] += ry2*rx4 - iy2*ix4;
         A2[i+8+1] += ry2*ix4 + iy2*rx4;
         A3[i+8] += ry3*rx4 - iy3*ix4;
         A3[i+8+1] += ry3*ix4 + iy3*rx4;
         A4[i+8] += ry4*rx4 - iy4*ix4;
         A4[i+8+1] += ry4*ix4 + iy4*rx4;
         A5[i+8] += ry5*rx4 - iy5*ix4;
         A5[i+8+1] += ry5*ix4 + iy5*rx4;
         A6[i+8] += ry6*rx4 - iy6*ix4;
         A6[i+8+1] += ry6*ix4 + iy6*rx4;
         A7[i+8] += ry7*rx4 - iy7*ix4;
         A7[i+8+1] += ry7*ix4 + iy7*rx4;
         A8[i+8] += ry8*rx4 - iy8*ix4;
         A8[i+8+1] += ry8*ix4 + iy8*rx4;
         A9[i+8] += ry9*rx4 - iy9*ix4;
         A9[i+8+1] += ry9*ix4 + iy9*rx4;
         A10[i+8] += ry10*rx4 - iy10*ix4;
         A10[i+8+1] += ry10*ix4 + iy10*rx4;
         A11[i+8] += ry11*rx4 - iy11*ix4;
         A11[i+8+1] += ry11*ix4 + iy11*rx4;
         A0[i+10] += ry0*rx5 - iy0*ix5;
         A0[i+10+1] += ry0*ix5 + iy0*rx5;
         A1[i+10] += ry1*rx5 - iy1*ix5;
         A1[i+10+1] += ry1*ix5 + iy1*rx5;
         A2[i+10] += ry2*rx5 - iy2*ix5;
         A2[i+10+1] += ry2*ix5 + iy2*rx5;
         A3[i+10] += ry3*rx5 - iy3*ix5;
         A3[i+10+1] += ry3*ix5 + iy3*rx5;
         A4[i+10] += ry4*rx5 - iy4*ix5;
         A4[i+10+1] += ry4*ix5 + iy4*rx5;
         A5[i+10] += ry5*rx5 - iy5*ix5;
         A5[i+10+1] += ry5*ix5 + iy5*rx5;
         A6[i+10] += ry6*rx5 - iy6*ix5;
         A6[i+10+1] += ry6*ix5 + iy6*rx5;
         A7[i+10] += ry7*rx5 - iy7*ix5;
         A7[i+10+1] += ry7*ix5 + iy7*rx5;
         A8[i+10] += ry8*rx5 - iy8*ix5;
         A8[i+10+1] += ry8*ix5 + iy8*rx5;
         A9[i+10] += ry9*rx5 - iy9*ix5;
         A9[i+10+1] += ry9*ix5 + iy9*rx5;
         A10[i+10] += ry10*rx5 - iy10*ix5;
         A10[i+10+1] += ry10*ix5 + iy10*rx5;
         A11[i+10] += ry11*rx5 - iy11*ix5;
         A11[i+10+1] += ry11*ix5 + iy11*rx5;
         A0[i+12] += ry0*rx6 - iy0*ix6;
         A0[i+12+1] += ry0*ix6 + iy0*rx6;
         A1[i+12] += ry1*rx6 - iy1*ix6;
         A1[i+12+1] += ry1*ix6 + iy1*rx6;
         A2[i+12] += ry2*rx6 - iy2*ix6;
         A2[i+12+1] += ry2*ix6 + iy2*rx6;
         A3[i+12] += ry3*rx6 - iy3*ix6;
         A3[i+12+1] += ry3*ix6 + iy3*rx6;
         A4[i+12] += ry4*rx6 - iy4*ix6;
         A4[i+12+1] += ry4*ix6 + iy4*rx6;
         A5[i+12] += ry5*rx6 - iy5*ix6;
         A5[i+12+1] += ry5*ix6 + iy5*rx6;
         A6[i+12] += ry6*rx6 - iy6*ix6;
         A6[i+12+1] += ry6*ix6 + iy6*rx6;
         A7[i+12] += ry7*rx6 - iy7*ix6;
         A7[i+12+1] += ry7*ix6 + iy7*rx6;
         A8[i+12] += ry8*rx6 - iy8*ix6;
         A8[i+12+1] += ry8*ix6 + iy8*rx6;
         A9[i+12] += ry9*rx6 - iy9*ix6;
         A9[i+12+1] += ry9*ix6 + iy9*rx6;
         A10[i+12] += ry10*rx6 - iy10*ix6;
         A10[i+12+1] += ry10*ix6 + iy10*rx6;
         A11[i+12] += ry11*rx6 - iy11*ix6;
         A11[i+12+1] += ry11*ix6 + iy11*rx6;
         A0[i+14] += ry0*rx7 - iy0*ix7;
         A0[i+14+1] += ry0*ix7 + iy0*rx7;
         A1[i+14] += ry1*rx7 - iy1*ix7;
         A1[i+14+1] += ry1*ix7 + iy1*rx7;
         A2[i+14] += ry2*rx7 - iy2*ix7;
         A2[i+14+1] += ry2*ix7 + iy2*rx7;
         A3[i+14] += ry3*rx7 - iy3*ix7;
         A3[i+14+1] += ry3*ix7 + iy3*rx7;
         A4[i+14] += ry4*rx7 - iy4*ix7;
         A4[i+14+1] += ry4*ix7 + iy4*rx7;
         A5[i+14] += ry5*rx7 - iy5*ix7;
         A5[i+14+1] += ry5*ix7 + iy5*rx7;
         A6[i+14] += ry6*rx7 - iy6*ix7;
         A6[i+14+1] += ry6*ix7 + iy6*rx7;
         A7[i+14] += ry7*rx7 - iy7*ix7;
         A7[i+14+1] += ry7*ix7 + iy7*rx7;
         A8[i+14] += ry8*rx7 - iy8*ix7;
         A8[i+14+1] += ry8*ix7 + iy8*rx7;
         A9[i+14] += ry9*rx7 - iy9*ix7;
         A9[i+14+1] += ry9*ix7 + iy9*rx7;
         A10[i+14] += ry10*rx7 - iy10*ix7;
         A10[i+14+1] += ry10*ix7 + iy10*rx7;
         A11[i+14] += ry11*rx7 - iy11*ix7;
         A11[i+14+1] += ry11*ix7 + iy11*rx7;
         A0[i+16] += ry0*rx8 - iy0*ix8;
         prefA(A0);
         A0[i+16+1] += ry0*ix8 + iy0*rx8;
         A1[i+16] += ry1*rx8 - iy1*ix8;
         prefA(A1);
         A1[i+16+1] += ry1*ix8 + iy1*rx8;
         A2[i+16] += ry2*rx8 - iy2*ix8;
         prefA(A2);
         A2[i+16+1] += ry2*ix8 + iy2*rx8;
         A3[i+16] += ry3*rx8 - iy3*ix8;
         prefA(A3);
         A3[i+16+1] += ry3*ix8 + iy3*rx8;
         A4[i+16] += ry4*rx8 - iy4*ix8;
         prefA(A4);
         A4[i+16+1] += ry4*ix8 + iy4*rx8;
         A5[i+16] += ry5*rx8 - iy5*ix8;
         prefA(A5);
         A5[i+16+1] += ry5*ix8 + iy5*rx8;
         A6[i+16] += ry6*rx8 - iy6*ix8;
         prefA(A6);
         A6[i+16+1] += ry6*ix8 + iy6*rx8;
         A7[i+16] += ry7*rx8 - iy7*ix8;
         prefA(A7);
         A7[i+16+1] += ry7*ix8 + iy7*rx8;
         A8[i+16] += ry8*rx8 - iy8*ix8;
         prefA(A8);
         A8[i+16+1] += ry8*ix8 + iy8*rx8;
         A9[i+16] += ry9*rx8 - iy9*ix8;
         prefA(A9);
         A9[i+16+1] += ry9*ix8 + iy9*rx8;
         A10[i+16] += ry10*rx8 - iy10*ix8;
         prefA(A10);
         A10[i+16+1] += ry10*ix8 + iy10*rx8;
         A11[i+16] += ry11*rx8 - iy11*ix8;
         prefA(A11);
         A11[i+16+1] += ry11*ix8 + iy11*rx8;
         A0[i+18] += ry0*rx9 - iy0*ix9;
         A0[i+18+1] += ry0*ix9 + iy0*rx9;
         A1[i+18] += ry1*rx9 - iy1*ix9;
         A1[i+18+1] += ry1*ix9 + iy1*rx9;
         A2[i+18] += ry2*rx9 - iy2*ix9;
         A2[i+18+1] += ry2*ix9 + iy2*rx9;
         A3[i+18] += ry3*rx9 - iy3*ix9;
         A3[i+18+1] += ry3*ix9 + iy3*rx9;
         A4[i+18] += ry4*rx9 - iy4*ix9;
         A4[i+18+1] += ry4*ix9 + iy4*rx9;
         A5[i+18] += ry5*rx9 - iy5*ix9;
         A5[i+18+1] += ry5*ix9 + iy5*rx9;
         A6[i+18] += ry6*rx9 - iy6*ix9;
         A6[i+18+1] += ry6*ix9 + iy6*rx9;
         A7[i+18] += ry7*rx9 - iy7*ix9;
         A7[i+18+1] += ry7*ix9 + iy7*rx9;
         A8[i+18] += ry8*rx9 - iy8*ix9;
         A8[i+18+1] += ry8*ix9 + iy8*rx9;
         A9[i+18] += ry9*rx9 - iy9*ix9;
         A9[i+18+1] += ry9*ix9 + iy9*rx9;
         A10[i+18] += ry10*rx9 - iy10*ix9;
         A10[i+18+1] += ry10*ix9 + iy10*rx9;
         A11[i+18] += ry11*rx9 - iy11*ix9;
         A11[i+18+1] += ry11*ix9 + iy11*rx9;
         A0[i+20] += ry0*rx10 - iy0*ix10;
         A0[i+20+1] += ry0*ix10 + iy0*rx10;
         A1[i+20] += ry1*rx10 - iy1*ix10;
         A1[i+20+1] += ry1*ix10 + iy1*rx10;
         A2[i+20] += ry2*rx10 - iy2*ix10;
         A2[i+20+1] += ry2*ix10 + iy2*rx10;
         A3[i+20] += ry3*rx10 - iy3*ix10;
         A3[i+20+1] += ry3*ix10 + iy3*rx10;
         A4[i+20] += ry4*rx10 - iy4*ix10;
         A4[i+20+1] += ry4*ix10 + iy4*rx10;
         A5[i+20] += ry5*rx10 - iy5*ix10;
         A5[i+20+1] += ry5*ix10 + iy5*rx10;
         A6[i+20] += ry6*rx10 - iy6*ix10;
         A6[i+20+1] += ry6*ix10 + iy6*rx10;
         A7[i+20] += ry7*rx10 - iy7*ix10;
         A7[i+20+1] += ry7*ix10 + iy7*rx10;
         A8[i+20] += ry8*rx10 - iy8*ix10;
         A8[i+20+1] += ry8*ix10 + iy8*rx10;
         A9[i+20] += ry9*rx10 - iy9*ix10;
         A9[i+20+1] += ry9*ix10 + iy9*rx10;
         A10[i+20] += ry10*rx10 - iy10*ix10;
         A10[i+20+1] += ry10*ix10 + iy10*rx10;
         A11[i+20] += ry11*rx10 - iy11*ix10;
         A11[i+20+1] += ry11*ix10 + iy11*rx10;
         A0[i+22] += ry0*rx11 - iy0*ix11;
         A0[i+22+1] += ry0*ix11 + iy0*rx11;
         A1[i+22] += ry1*rx11 - iy1*ix11;
         A1[i+22+1] += ry1*ix11 + iy1*rx11;
         A2[i+22] += ry2*rx11 - iy2*ix11;
         A2[i+22+1] += ry2*ix11 + iy2*rx11;
         A3[i+22] += ry3*rx11 - iy3*ix11;
         A3[i+22+1] += ry3*ix11 + iy3*rx11;
         A4[i+22] += ry4*rx11 - iy4*ix11;
         A4[i+22+1] += ry4*ix11 + iy4*rx11;
         A5[i+22] += ry5*rx11 - iy5*ix11;
         A5[i+22+1] += ry5*ix11 + iy5*rx11;
         A6[i+22] += ry6*rx11 - iy6*ix11;
         A6[i+22+1] += ry6*ix11 + iy6*rx11;
         A7[i+22] += ry7*rx11 - iy7*ix11;
         A7[i+22+1] += ry7*ix11 + iy7*rx11;
         A8[i+22] += ry8*rx11 - iy8*ix11;
         A8[i+22+1] += ry8*ix11 + iy8*rx11;
         A9[i+22] += ry9*rx11 - iy9*ix11;
         A9[i+22+1] += ry9*ix11 + iy9*rx11;
         A10[i+22] += ry10*rx11 - iy10*ix11;
         A10[i+22+1] += ry10*ix11 + iy10*rx11;
         A11[i+22] += ry11*rx11 - iy11*ix11;
         A11[i+22+1] += ry11*ix11 + iy11*rx11;
         A0[i+24] += ry0*rx12 - iy0*ix12;
         A0[i+24+1] += ry0*ix12 + iy0*rx12;
         A1[i+24] += ry1*rx12 - iy1*ix12;
         A1[i+24+1] += ry1*ix12 + iy1*rx12;
         A2[i+24] += ry2*rx12 - iy2*ix12;
         A2[i+24+1] += ry2*ix12 + iy2*rx12;
         A3[i+24] += ry3*rx12 - iy3*ix12;
         A3[i+24+1] += ry3*ix12 + iy3*rx12;
         A4[i+24] += ry4*rx12 - iy4*ix12;
         A4[i+24+1] += ry4*ix12 + iy4*rx12;
         A5[i+24] += ry5*rx12 - iy5*ix12;
         A5[i+24+1] += ry5*ix12 + iy5*rx12;
         A6[i+24] += ry6*rx12 - iy6*ix12;
         A6[i+24+1] += ry6*ix12 + iy6*rx12;
         A7[i+24] += ry7*rx12 - iy7*ix12;
         A7[i+24+1] += ry7*ix12 + iy7*rx12;
         A8[i+24] += ry8*rx12 - iy8*ix12;
         A8[i+24+1] += ry8*ix12 + iy8*rx12;
         A9[i+24] += ry9*rx12 - iy9*ix12;
         A9[i+24+1] += ry9*ix12 + iy9*rx12;
         A10[i+24] += ry10*rx12 - iy10*ix12;
         A10[i+24+1] += ry10*ix12 + iy10*rx12;
         A11[i+24] += ry11*rx12 - iy11*ix12;
         A11[i+24+1] += ry11*ix12 + iy11*rx12;
         A0[i+26] += ry0*rx13 - iy0*ix13;
         A0[i+26+1] += ry0*ix13 + iy0*rx13;
         A1[i+26] += ry1*rx13 - iy1*ix13;
         A1[i+26+1] += ry1*ix13 + iy1*rx13;
         A2[i+26] += ry2*rx13 - iy2*ix13;
         A2[i+26+1] += ry2*ix13 + iy2*rx13;
         A3[i+26] += ry3*rx13 - iy3*ix13;
         A3[i+26+1] += ry3*ix13 + iy3*rx13;
         A4[i+26] += ry4*rx13 - iy4*ix13;
         A4[i+26+1] += ry4*ix13 + iy4*rx13;
         A5[i+26] += ry5*rx13 - iy5*ix13;
         A5[i+26+1] += ry5*ix13 + iy5*rx13;
         A6[i+26] += ry6*rx13 - iy6*ix13;
         A6[i+26+1] += ry6*ix13 + iy6*rx13;
         A7[i+26] += ry7*rx13 - iy7*ix13;
         A7[i+26+1] += ry7*ix13 + iy7*rx13;
         A8[i+26] += ry8*rx13 - iy8*ix13;
         A8[i+26+1] += ry8*ix13 + iy8*rx13;
         A9[i+26] += ry9*rx13 - iy9*ix13;
         A9[i+26+1] += ry9*ix13 + iy9*rx13;
         A10[i+26] += ry10*rx13 - iy10*ix13;
         A10[i+26+1] += ry10*ix13 + iy10*rx13;
         A11[i+26] += ry11*rx13 - iy11*ix13;
         A11[i+26+1] += ry11*ix13 + iy11*rx13;
         A0[i+28] += ry0*rx14 - iy0*ix14;
         A0[i+28+1] += ry0*ix14 + iy0*rx14;
         A1[i+28] += ry1*rx14 - iy1*ix14;
         A1[i+28+1] += ry1*ix14 + iy1*rx14;
         A2[i+28] += ry2*rx14 - iy2*ix14;
         A2[i+28+1] += ry2*ix14 + iy2*rx14;
         A3[i+28] += ry3*rx14 - iy3*ix14;
         A3[i+28+1] += ry3*ix14 + iy3*rx14;
         A4[i+28] += ry4*rx14 - iy4*ix14;
         A4[i+28+1] += ry4*ix14 + iy4*rx14;
         A5[i+28] += ry5*rx14 - iy5*ix14;
         A5[i+28+1] += ry5*ix14 + iy5*rx14;
         A6[i+28] += ry6*rx14 - iy6*ix14;
         A6[i+28+1] += ry6*ix14 + iy6*rx14;
         A7[i+28] += ry7*rx14 - iy7*ix14;
         A7[i+28+1] += ry7*ix14 + iy7*rx14;
         A8[i+28] += ry8*rx14 - iy8*ix14;
         A8[i+28+1] += ry8*ix14 + iy8*rx14;
         A9[i+28] += ry9*rx14 - iy9*ix14;
         A9[i+28+1] += ry9*ix14 + iy9*rx14;
         A10[i+28] += ry10*rx14 - iy10*ix14;
         A10[i+28+1] += ry10*ix14 + iy10*rx14;
         A11[i+28] += ry11*rx14 - iy11*ix14;
         A11[i+28+1] += ry11*ix14 + iy11*rx14;
         A0[i+30] += ry0*rx15 - iy0*ix15;
         A0[i+30+1] += ry0*ix15 + iy0*rx15;
         A1[i+30] += ry1*rx15 - iy1*ix15;
         A1[i+30+1] += ry1*ix15 + iy1*rx15;
         A2[i+30] += ry2*rx15 - iy2*ix15;
         A2[i+30+1] += ry2*ix15 + iy2*rx15;
         A3[i+30] += ry3*rx15 - iy3*ix15;
         A3[i+30+1] += ry3*ix15 + iy3*rx15;
         A4[i+30] += ry4*rx15 - iy4*ix15;
         A4[i+30+1] += ry4*ix15 + iy4*rx15;
         A5[i+30] += ry5*rx15 - iy5*ix15;
         A5[i+30+1] += ry5*ix15 + iy5*rx15;
         A6[i+30] += ry6*rx15 - iy6*ix15;
         A6[i+30+1] += ry6*ix15 + iy6*rx15;
         A7[i+30] += ry7*rx15 - iy7*ix15;
         A7[i+30+1] += ry7*ix15 + iy7*rx15;
         A8[i+30] += ry8*rx15 - iy8*ix15;
         A8[i+30+1] += ry8*ix15 + iy8*rx15;
         A9[i+30] += ry9*rx15 - iy9*ix15;
         A9[i+30+1] += ry9*ix15 + iy9*rx15;
         A10[i+30] += ry10*rx15 - iy10*ix15;
         A10[i+30+1] += ry10*ix15 + iy10*rx15;
         A11[i+30] += ry11*rx15 - iy11*ix15;
         A11[i+30+1] += ry11*ix15 + iy11*rx15;
         A0[i+32] += ry0*rx16 - iy0*ix16;
         prefA(A0);
         A0[i+32+1] += ry0*ix16 + iy0*rx16;
         A1[i+32] += ry1*rx16 - iy1*ix16;
         prefA(A1);
         A1[i+32+1] += ry1*ix16 + iy1*rx16;
         A2[i+32] += ry2*rx16 - iy2*ix16;
         prefA(A2);
         A2[i+32+1] += ry2*ix16 + iy2*rx16;
         A3[i+32] += ry3*rx16 - iy3*ix16;
         prefA(A3);
         A3[i+32+1] += ry3*ix16 + iy3*rx16;
         A4[i+32] += ry4*rx16 - iy4*ix16;
         prefA(A4);
         A4[i+32+1] += ry4*ix16 + iy4*rx16;
         A5[i+32] += ry5*rx16 - iy5*ix16;
         prefA(A5);
         A5[i+32+1] += ry5*ix16 + iy5*rx16;
         A6[i+32] += ry6*rx16 - iy6*ix16;
         prefA(A6);
         A6[i+32+1] += ry6*ix16 + iy6*rx16;
         A7[i+32] += ry7*rx16 - iy7*ix16;
         prefA(A7);
         A7[i+32+1] += ry7*ix16 + iy7*rx16;
         A8[i+32] += ry8*rx16 - iy8*ix16;
         prefA(A8);
         A8[i+32+1] += ry8*ix16 + iy8*rx16;
         A9[i+32] += ry9*rx16 - iy9*ix16;
         prefA(A9);
         A9[i+32+1] += ry9*ix16 + iy9*rx16;
         A10[i+32] += ry10*rx16 - iy10*ix16;
         prefA(A10);
         A10[i+32+1] += ry10*ix16 + iy10*rx16;
         A11[i+32] += ry11*rx16 - iy11*ix16;
         prefA(A11);
         A11[i+32+1] += ry11*ix16 + iy11*rx16;
         A0[i+34] += ry0*rx17 - iy0*ix17;
         A0[i+34+1] += ry0*ix17 + iy0*rx17;
         A1[i+34] += ry1*rx17 - iy1*ix17;
         A1[i+34+1] += ry1*ix17 + iy1*rx17;
         A2[i+34] += ry2*rx17 - iy2*ix17;
         A2[i+34+1] += ry2*ix17 + iy2*rx17;
         A3[i+34] += ry3*rx17 - iy3*ix17;
         A3[i+34+1] += ry3*ix17 + iy3*rx17;
         A4[i+34] += ry4*rx17 - iy4*ix17;
         A4[i+34+1] += ry4*ix17 + iy4*rx17;
         A5[i+34] += ry5*rx17 - iy5*ix17;
         A5[i+34+1] += ry5*ix17 + iy5*rx17;
         A6[i+34] += ry6*rx17 - iy6*ix17;
         A6[i+34+1] += ry6*ix17 + iy6*rx17;
         A7[i+34] += ry7*rx17 - iy7*ix17;
         A7[i+34+1] += ry7*ix17 + iy7*rx17;
         A8[i+34] += ry8*rx17 - iy8*ix17;
         A8[i+34+1] += ry8*ix17 + iy8*rx17;
         A9[i+34] += ry9*rx17 - iy9*ix17;
         A9[i+34+1] += ry9*ix17 + iy9*rx17;
         A10[i+34] += ry10*rx17 - iy10*ix17;
         A10[i+34+1] += ry10*ix17 + iy10*rx17;
         A11[i+34] += ry11*rx17 - iy11*ix17;
         A11[i+34+1] += ry11*ix17 + iy11*rx17;
         A0[i+36] += ry0*rx18 - iy0*ix18;
         A0[i+36+1] += ry0*ix18 + iy0*rx18;
         A1[i+36] += ry1*rx18 - iy1*ix18;
         A1[i+36+1] += ry1*ix18 + iy1*rx18;
         A2[i+36] += ry2*rx18 - iy2*ix18;
         A2[i+36+1] += ry2*ix18 + iy2*rx18;
         A3[i+36] += ry3*rx18 - iy3*ix18;
         A3[i+36+1] += ry3*ix18 + iy3*rx18;
         A4[i+36] += ry4*rx18 - iy4*ix18;
         A4[i+36+1] += ry4*ix18 + iy4*rx18;
         A5[i+36] += ry5*rx18 - iy5*ix18;
         A5[i+36+1] += ry5*ix18 + iy5*rx18;
         A6[i+36] += ry6*rx18 - iy6*ix18;
         A6[i+36+1] += ry6*ix18 + iy6*rx18;
         A7[i+36] += ry7*rx18 - iy7*ix18;
         A7[i+36+1] += ry7*ix18 + iy7*rx18;
         A8[i+36] += ry8*rx18 - iy8*ix18;
         A8[i+36+1] += ry8*ix18 + iy8*rx18;
         A9[i+36] += ry9*rx18 - iy9*ix18;
         A9[i+36+1] += ry9*ix18 + iy9*rx18;
         A10[i+36] += ry10*rx18 - iy10*ix18;
         A10[i+36+1] += ry10*ix18 + iy10*rx18;
         A11[i+36] += ry11*rx18 - iy11*ix18;
         A11[i+36+1] += ry11*ix18 + iy11*rx18;
         A0[i+38] += ry0*rx19 - iy0*ix19;
         A0[i+38+1] += ry0*ix19 + iy0*rx19;
         A1[i+38] += ry1*rx19 - iy1*ix19;
         A1[i+38+1] += ry1*ix19 + iy1*rx19;
         A2[i+38] += ry2*rx19 - iy2*ix19;
         A2[i+38+1] += ry2*ix19 + iy2*rx19;
         A3[i+38] += ry3*rx19 - iy3*ix19;
         A3[i+38+1] += ry3*ix19 + iy3*rx19;
         A4[i+38] += ry4*rx19 - iy4*ix19;
         A4[i+38+1] += ry4*ix19 + iy4*rx19;
         A5[i+38] += ry5*rx19 - iy5*ix19;
         A5[i+38+1] += ry5*ix19 + iy5*rx19;
         A6[i+38] += ry6*rx19 - iy6*ix19;
         A6[i+38+1] += ry6*ix19 + iy6*rx19;
         A7[i+38] += ry7*rx19 - iy7*ix19;
         A7[i+38+1] += ry7*ix19 + iy7*rx19;
         A8[i+38] += ry8*rx19 - iy8*ix19;
         A8[i+38+1] += ry8*ix19 + iy8*rx19;
         A9[i+38] += ry9*rx19 - iy9*ix19;
         A9[i+38+1] += ry9*ix19 + iy9*rx19;
         A10[i+38] += ry10*rx19 - iy10*ix19;
         A10[i+38+1] += ry10*ix19 + iy10*rx19;
         A11[i+38] += ry11*rx19 - iy11*ix19;
         A11[i+38+1] += ry11*ix19 + iy11*rx19;
         A0[i+40] += ry0*rx20 - iy0*ix20;
         A0[i+40+1] += ry0*ix20 + iy0*rx20;
         A1[i+40] += ry1*rx20 - iy1*ix20;
         A1[i+40+1] += ry1*ix20 + iy1*rx20;
         A2[i+40] += ry2*rx20 - iy2*ix20;
         A2[i+40+1] += ry2*ix20 + iy2*rx20;
         A3[i+40] += ry3*rx20 - iy3*ix20;
         A3[i+40+1] += ry3*ix20 + iy3*rx20;
         A4[i+40] += ry4*rx20 - iy4*ix20;
         A4[i+40+1] += ry4*ix20 + iy4*rx20;
         A5[i+40] += ry5*rx20 - iy5*ix20;
         A5[i+40+1] += ry5*ix20 + iy5*rx20;
         A6[i+40] += ry6*rx20 - iy6*ix20;
         A6[i+40+1] += ry6*ix20 + iy6*rx20;
         A7[i+40] += ry7*rx20 - iy7*ix20;
         A7[i+40+1] += ry7*ix20 + iy7*rx20;
         A8[i+40] += ry8*rx20 - iy8*ix20;
         A8[i+40+1] += ry8*ix20 + iy8*rx20;
         A9[i+40] += ry9*rx20 - iy9*ix20;
         A9[i+40+1] += ry9*ix20 + iy9*rx20;
         A10[i+40] += ry10*rx20 - iy10*ix20;
         A10[i+40+1] += ry10*ix20 + iy10*rx20;
         A11[i+40] += ry11*rx20 - iy11*ix20;
         A11[i+40+1] += ry11*ix20 + iy11*rx20;
         A0[i+42] += ry0*rx21 - iy0*ix21;
         A0[i+42+1] += ry0*ix21 + iy0*rx21;
         A1[i+42] += ry1*rx21 - iy1*ix21;
         A1[i+42+1] += ry1*ix21 + iy1*rx21;
         A2[i+42] += ry2*rx21 - iy2*ix21;
         A2[i+42+1] += ry2*ix21 + iy2*rx21;
         A3[i+42] += ry3*rx21 - iy3*ix21;
         A3[i+42+1] += ry3*ix21 + iy3*rx21;
         A4[i+42] += ry4*rx21 - iy4*ix21;
         A4[i+42+1] += ry4*ix21 + iy4*rx21;
         A5[i+42] += ry5*rx21 - iy5*ix21;
         A5[i+42+1] += ry5*ix21 + iy5*rx21;
         A6[i+42] += ry6*rx21 - iy6*ix21;
         A6[i+42+1] += ry6*ix21 + iy6*rx21;
         A7[i+42] += ry7*rx21 - iy7*ix21;
         A7[i+42+1] += ry7*ix21 + iy7*rx21;
         A8[i+42] += ry8*rx21 - iy8*ix21;
         A8[i+42+1] += ry8*ix21 + iy8*rx21;
         A9[i+42] += ry9*rx21 - iy9*ix21;
         A9[i+42+1] += ry9*ix21 + iy9*rx21;
         A10[i+42] += ry10*rx21 - iy10*ix21;
         A10[i+42+1] += ry10*ix21 + iy10*rx21;
         A11[i+42] += ry11*rx21 - iy11*ix21;
         A11[i+42+1] += ry11*ix21 + iy11*rx21;
         A0[i+44] += ry0*rx22 - iy0*ix22;
         A0[i+44+1] += ry0*ix22 + iy0*rx22;
         A1[i+44] += ry1*rx22 - iy1*ix22;
         A1[i+44+1] += ry1*ix22 + iy1*rx22;
         A2[i+44] += ry2*rx22 - iy2*ix22;
         A2[i+44+1] += ry2*ix22 + iy2*rx22;
         A3[i+44] += ry3*rx22 - iy3*ix22;
         A3[i+44+1] += ry3*ix22 + iy3*rx22;
         A4[i+44] += ry4*rx22 - iy4*ix22;
         A4[i+44+1] += ry4*ix22 + iy4*rx22;
         A5[i+44] += ry5*rx22 - iy5*ix22;
         A5[i+44+1] += ry5*ix22 + iy5*rx22;
         A6[i+44] += ry6*rx22 - iy6*ix22;
         A6[i+44+1] += ry6*ix22 + iy6*rx22;
         A7[i+44] += ry7*rx22 - iy7*ix22;
         A7[i+44+1] += ry7*ix22 + iy7*rx22;
         A8[i+44] += ry8*rx22 - iy8*ix22;
         A8[i+44+1] += ry8*ix22 + iy8*rx22;
         A9[i+44] += ry9*rx22 - iy9*ix22;
         A9[i+44+1] += ry9*ix22 + iy9*rx22;
         A10[i+44] += ry10*rx22 - iy10*ix22;
         A10[i+44+1] += ry10*ix22 + iy10*rx22;
         A11[i+44] += ry11*rx22 - iy11*ix22;
         A11[i+44+1] += ry11*ix22 + iy11*rx22;
         A0[i+46] += ry0*rx23 - iy0*ix23;
         A0[i+46+1] += ry0*ix23 + iy0*rx23;
         A1[i+46] += ry1*rx23 - iy1*ix23;
         A1[i+46+1] += ry1*ix23 + iy1*rx23;
         A2[i+46] += ry2*rx23 - iy2*ix23;
         A2[i+46+1] += ry2*ix23 + iy2*rx23;
         A3[i+46] += ry3*rx23 - iy3*ix23;
         A3[i+46+1] += ry3*ix23 + iy3*rx23;
         A4[i+46] += ry4*rx23 - iy4*ix23;
         A4[i+46+1] += ry4*ix23 + iy4*rx23;
         A5[i+46] += ry5*rx23 - iy5*ix23;
         A5[i+46+1] += ry5*ix23 + iy5*rx23;
         A6[i+46] += ry6*rx23 - iy6*ix23;
         A6[i+46+1] += ry6*ix23 + iy6*rx23;
         A7[i+46] += ry7*rx23 - iy7*ix23;
         A7[i+46+1] += ry7*ix23 + iy7*rx23;
         A8[i+46] += ry8*rx23 - iy8*ix23;
         A8[i+46+1] += ry8*ix23 + iy8*rx23;
         A9[i+46] += ry9*rx23 - iy9*ix23;
         A9[i+46+1] += ry9*ix23 + iy9*rx23;
         A10[i+46] += ry10*rx23 - iy10*ix23;
         A10[i+46+1] += ry10*ix23 + iy10*rx23;
         A11[i+46] += ry11*rx23 - iy11*ix23;
         A11[i+46+1] += ry11*ix23 + iy11*rx23;
      }
      for (i=M24; i < M; i += 2)
      {
         const register float rx0 = X[i], ix0 = X[i+1];
         A0[i] += ry0 * rx0 - iy0 * ix0;
         A0[i+1] += ry0 * ix0 + iy0 * rx0;
         A1[i] += ry1 * rx0 - iy1 * ix0;
         A1[i+1] += ry1 * ix0 + iy1 * rx0;
         A2[i] += ry2 * rx0 - iy2 * ix0;
         A2[i+1] += ry2 * ix0 + iy2 * rx0;
         A3[i] += ry3 * rx0 - iy3 * ix0;
         A3[i+1] += ry3 * ix0 + iy3 * rx0;
         A4[i] += ry4 * rx0 - iy4 * ix0;
         A4[i+1] += ry4 * ix0 + iy4 * rx0;
         A5[i] += ry5 * rx0 - iy5 * ix0;
         A5[i+1] += ry5 * ix0 + iy5 * rx0;
         A6[i] += ry6 * rx0 - iy6 * ix0;
         A6[i+1] += ry6 * ix0 + iy6 * rx0;
         A7[i] += ry7 * rx0 - iy7 * ix0;
         A7[i+1] += ry7 * ix0 + iy7 * rx0;
         A8[i] += ry8 * rx0 - iy8 * ix0;
         A8[i+1] += ry8 * ix0 + iy8 * rx0;
         A9[i] += ry9 * rx0 - iy9 * ix0;
         A9[i+1] += ry9 * ix0 + iy9 * rx0;
         A10[i] += ry10 * rx0 - iy10 * ix0;
         A10[i+1] += ry10 * ix0 + iy10 * rx0;
         A11[i] += ry11 * rx0 - iy11 * ix0;
         A11[i+1] += ry11 * ix0 + iy11 * rx0;
      }
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N12; j; j--, A += lda, Y += 2)
   {
      float *A0=A;
      const register float ry0=Y[0], iy0=Y[1];
      ATL_INT i;
      prefY(Y+24+24-1);
      for (i=0; i < M24; i += 48)
      {
         const register float rx0=X[i+0], ix0=X[i+1], rx1=X[i+2], ix1=X[i+3],
                              rx2=X[i+4], ix2=X[i+5], rx3=X[i+6], ix3=X[i+7],
                              rx4=X[i+8], ix4=X[i+9], rx5=X[i+10], ix5=X[i+11],
                              rx6=X[i+12], ix6=X[i+13], rx7=X[i+14],
                              ix7=X[i+15], rx8=X[i+16], ix8=X[i+17],
                              rx9=X[i+18], ix9=X[i+19], rx10=X[i+20],
                              ix10=X[i+21], rx11=X[i+22], ix11=X[i+23],
                              rx12=X[i+24], ix12=X[i+25], rx13=X[i+26],
                              ix13=X[i+27], rx14=X[i+28], ix14=X[i+29],
                              rx15=X[i+30], ix15=X[i+31], rx16=X[i+32],
                              ix16=X[i+33], rx17=X[i+34], ix17=X[i+35],
                              rx18=X[i+36], ix18=X[i+37], rx19=X[i+38],
                              ix19=X[i+39], rx20=X[i+40], ix20=X[i+41],
                              rx21=X[i+42], ix21=X[i+43], rx22=X[i+44],
                              ix22=X[i+45], rx23=X[i+46], ix23=X[i+47];

         prefX(X);
         A0[i+0] += ry0*rx0 - iy0*ix0;
         prefA(A0);
         A0[i+0+1] += ry0*ix0 + iy0*rx0;
         A0[i+2] += ry0*rx1 - iy0*ix1;
         A0[i+2+1] += ry0*ix1 + iy0*rx1;
         A0[i+4] += ry0*rx2 - iy0*ix2;
         A0[i+4+1] += ry0*ix2 + iy0*rx2;
         A0[i+6] += ry0*rx3 - iy0*ix3;
         A0[i+6+1] += ry0*ix3 + iy0*rx3;
         A0[i+8] += ry0*rx4 - iy0*ix4;
         A0[i+8+1] += ry0*ix4 + iy0*rx4;
         A0[i+10] += ry0*rx5 - iy0*ix5;
         A0[i+10+1] += ry0*ix5 + iy0*rx5;
         A0[i+12] += ry0*rx6 - iy0*ix6;
         A0[i+12+1] += ry0*ix6 + iy0*rx6;
         A0[i+14] += ry0*rx7 - iy0*ix7;
         A0[i+14+1] += ry0*ix7 + iy0*rx7;
         A0[i+16] += ry0*rx8 - iy0*ix8;
         prefA(A0);
         A0[i+16+1] += ry0*ix8 + iy0*rx8;
         A0[i+18] += ry0*rx9 - iy0*ix9;
         A0[i+18+1] += ry0*ix9 + iy0*rx9;
         A0[i+20] += ry0*rx10 - iy0*ix10;
         A0[i+20+1] += ry0*ix10 + iy0*rx10;
         A0[i+22] += ry0*rx11 - iy0*ix11;
         A0[i+22+1] += ry0*ix11 + iy0*rx11;
         A0[i+24] += ry0*rx12 - iy0*ix12;
         A0[i+24+1] += ry0*ix12 + iy0*rx12;
         A0[i+26] += ry0*rx13 - iy0*ix13;
         A0[i+26+1] += ry0*ix13 + iy0*rx13;
         A0[i+28] += ry0*rx14 - iy0*ix14;
         A0[i+28+1] += ry0*ix14 + iy0*rx14;
         A0[i+30] += ry0*rx15 - iy0*ix15;
         A0[i+30+1] += ry0*ix15 + iy0*rx15;
         A0[i+32] += ry0*rx16 - iy0*ix16;
         prefA(A0);
         A0[i+32+1] += ry0*ix16 + iy0*rx16;
         A0[i+34] += ry0*rx17 - iy0*ix17;
         A0[i+34+1] += ry0*ix17 + iy0*rx17;
         A0[i+36] += ry0*rx18 - iy0*ix18;
         A0[i+36+1] += ry0*ix18 + iy0*rx18;
         A0[i+38] += ry0*rx19 - iy0*ix19;
         A0[i+38+1] += ry0*ix19 + iy0*rx19;
         A0[i+40] += ry0*rx20 - iy0*ix20;
         A0[i+40+1] += ry0*ix20 + iy0*rx20;
         A0[i+42] += ry0*rx21 - iy0*ix21;
         A0[i+42+1] += ry0*ix21 + iy0*rx21;
         A0[i+44] += ry0*rx22 - iy0*ix22;
         A0[i+44+1] += ry0*ix22 + iy0*rx22;
         A0[i+46] += ry0*rx23 - iy0*ix23;
         A0[i+46+1] += ry0*ix23 + iy0*rx23;
      }
      for (i=M24; i < M; i += 2)
      {
         const register float rx0 = X[i], ix0 = X[i+1];
         A0[i] += ry0 * rx0 - iy0 * ix0;
         A0[i+1] += ry0 * ix0 + iy0 * rx0;
      }
   }
}
